
/* Chrysalide - Outil d'analyse de fichiers binaires
 * gbinarycursor.c - suivi de positions dans des panneaux de chargement
 *
 * Copyright (C) 2018 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "gbinarycursor.h"


#include "glinecursor-int.h"



/* Suivi de positions dans un panneau de chargement (instance) */
struct _GBinaryCursor
{
    GLineCursor parent;                     /* A laisser en premier        */

    vmpa2t addr;                            /* Position mémoire du curseur */

};

/* Suivi de positions dans un panneau de chargement (classe) */
struct _GBinaryCursorClass
{
    GLineCursorClass parent;                /* A laisser en premier        */

};


/* Procède à l'initialisation d'une classe de suivi de position. */
static void g_binary_cursor_class_init(GBinaryCursorClass *);

/* Procède à l'initialisation d'un suivi de positions. */
static void g_binary_cursor_init(GBinaryCursor *);

/* Supprime toutes les références externes. */
static void g_binary_cursor_dispose(GBinaryCursor *);

/* Procède à la libération totale de la mémoire. */
static void g_binary_cursor_finalize(GBinaryCursor *);

/* Compare deux suivis d'emplacements. */
static int g_binary_cursor_compare(const GBinaryCursor *, const GBinaryCursor *);

/* Détermine si un suivi d'emplacement est valide ou non. */
static bool g_binary_cursor_is_valid(const GBinaryCursor *);



/* Détermine le type du gestionnaire de largeurs associées aux lignes. */
G_DEFINE_TYPE(GBinaryCursor, g_binary_cursor, G_TYPE_LINE_CURSOR);


/******************************************************************************
*                                                                             *
*  Paramètres  : class = classe de composant GTK à initialiser.               *
*                                                                             *
*  Description : Procède à l'initialisation d'une classe de suivi de position.*
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_binary_cursor_class_init(GBinaryCursorClass *class)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GLineCursorClass *line;                 /* Version parente de la classe*/

    object = G_OBJECT_CLASS(class);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_binary_cursor_dispose;
    object->finalize = (GObjectFinalizeFunc)g_binary_cursor_finalize;

    line = G_LINE_CURSOR_CLASS(class);

    line->compare = (compare_cursor_fc)g_binary_cursor_compare;
    line->is_valid = (is_cursor_valid_fc)g_binary_cursor_is_valid;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : cursor = composant GLib à initialiser.                       *
*                                                                             *
*  Description : Procède à l'initialisation d'un suivi de positions.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_binary_cursor_init(GBinaryCursor *cursor)
{
    init_vmpa(&cursor->addr, VMPA_NO_PHYSICAL, VMPA_NO_VIRTUAL);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : cursor = instance d'objet GLib à traiter.                    *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_binary_cursor_dispose(GBinaryCursor *cursor)
{
    G_OBJECT_CLASS(g_binary_cursor_parent_class)->dispose(G_OBJECT(cursor));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : cursor = instance d'objet GLib à traiter.                    *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_binary_cursor_finalize(GBinaryCursor *cursor)
{
    G_OBJECT_CLASS(g_binary_cursor_parent_class)->finalize(G_OBJECT(cursor));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Crée un nouveau suivi de positions dans un panneau.          *
*                                                                             *
*  Retour      : Instance de suivi en place.                                  *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GLineCursor *g_binary_cursor_new(void)
{
    GLineCursor *result;                    /* Instance à retourner        */

    result = g_object_new(G_TYPE_BINARY_CURSOR, NULL);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : cursor = premier suivi d'emplacement à comparer.             *
*                other  = second suivi d'emplacement à comparer.              *
*                                                                             *
*  Description : Compare deux suivis d'emplacements.                          *
*                                                                             *
*  Retour      : Bilan de la comparaison.                                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static int g_binary_cursor_compare(const GBinaryCursor *cursor, const GBinaryCursor *other)
{
    int result;                             /* Bilan à renvoyer            */

    result = cmp_vmpa(&cursor->addr, &other->addr);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : cursor = suivi d'emplacement à consulter.                    *
*                                                                             *
*  Description : Détermine si un suivi d'emplacement est valide ou non.       *
*                                                                             *
*  Retour      : Bilan de validité.                                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool g_binary_cursor_is_valid(const GBinaryCursor *cursor)
{
    bool result;                            /* Bilan à renvoyer            */

    result = !is_invalid_vmpa(&cursor->addr);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : cursor = suivi de positions à mettre à jour.                 *
*                addr   = emplacement dans le binaire visé.                   *
*                                                                             *
*  Description : Met à jour la position suivi dans un panneau de chargement.  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_binary_cursor_update(GBinaryCursor *cursor, const vmpa2t *addr)
{
    copy_vmpa(&cursor->addr, addr);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : cursor = suivi de positions à consulter.                     *
*                addr   = emplacement dans le binaire visé. [OUT]             *
*                                                                             *
*  Description : Transmet la position suivi dans un panneau de chargement.    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_binary_cursor_get_info(GBinaryCursor *cursor, vmpa2t *addr)
{
    copy_vmpa(addr, &cursor->addr);

}
